@@ -4,9 +4,12 @@
  *	     Copyright (C) 1994, 1995  Kazuhiko Yamamoto
  *
  * This software conforms GNU GENERAL PUBLIC LICENSE Version 2.
+ * The modification copyright also conforms GNU GENERAL PUBLIC LICENSE
+ * Version 2.
  *
  * Author:  Kazuhiko Yamamoto <kazu@is.aist-nara.ac.jp>
  * Created: December 8, 1994
+ * Revised: October  5, 1995
  *
  */
 
@@ -27,7 +30,7 @@
   ****************************************************************
 */
 
-static char version_message[] = "mewencode version 0.04 02/06/95 Kazuhiko Yamamoto";
+static char version_message[] = "mewencode version 0.07 10/06/95 Kazuhiko Yamamoto";
 
 #include "getopt.h"
 #include <stdio.h>
@@ -310,7 +313,6 @@ base64_encode(infile, outfile, text, length)
 }
 
 #define OOB -1
-#define PAD -2	
 
 static char base256[] = {
     OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB,
@@ -321,7 +323,7 @@ static char base256[] = {
     OOB,OOB,OOB,OOB, OOB,OOB,OOB,OOB, OOB,OOB,OOB, 62, OOB,OOB,OOB, 63,
       
   /*  0   1   2   3    4   5   6   7    8   9                =        */
-     52, 53, 54, 55,  56, 57, 58, 59,  60, 61,OOB,OOB, OOB,PAD,OOB,OOB,
+     52, 53, 54, 55,  56, 57, 58, 59,  60, 61,OOB,OOB, OOB,OOB,OOB,OOB,
       
   /*      A   B   C    D   E   F   G    H   I   J   K    L   M   N   O*/
     OOB,  0,  1,  2,   3,  4,  5,  6,   7,  8,  9, 10,  11, 12, 13, 14,
@@ -366,6 +368,8 @@ base64_decode(infile, outfile, text, length)
 	if ((c3 = GetChar(infile)) == EOF) {
 	    fprintf(stderr, "Warning: Base64 decoder saw premature EOF.\n");
 	    exit(ERROR);
+	} else if (c3 == PADDING) {
+	    break;
 	} else if ((c3 = base256[c3]) == OOB) {
 	    fprintf(stderr,
 		    "Warning: Base64 decoder saw an illegal character.\n");
@@ -373,24 +377,20 @@ base64_decode(infile, outfile, text, length)
 	}
 	
 	
-	if( c3 == PAD) {
-	    break;
-	} else 
-	    PutChar((((c2 & 0XF) << 4) | ((c3 & 0x3C) >> 2)), outfile, text);
-	    
+        PutChar((((c2 & 0XF) << 4) | ((c3 & 0x3C) >> 2)), outfile, text);
+	
 	if ((c4 = GetChar(infile)) == EOF) {
 	    fprintf(stderr, "Warning: Base64 decoder saw premature EOF.\n");
 	    exit(ERROR);
-	} else if ((c4 = base256[c4]) == OOB) { 
+	} else if (c4 == PADDING) {
+	    break;
+	} else if ((c4 = base256[c4]) == OOB) {
 	    fprintf(stderr,
 		    "Warning: Base64 decoder saw an illegal character.\n");
 	    exit(ERROR);
 	}
 	    	    
-	if( c4 == PAD) {
-	    break;
-	} else 
-	    PutChar((((c3 & 0x03) << 6) | c4), outfile, text);
+        PutChar((((c3 & 0x03) << 6) | c4), outfile, text);
     }
 
     PutChar(EOF);
@@ -405,6 +405,7 @@ static char base16[] = "0123456789ABCDEF";
 #define EQ   '='
 #define TAB   9
 #define SP   32
+#define DOT  '.'
 #define DEL 127
 #define softbreak(stream) {putc(EQ, stream); putc(LF, stream);}
 
@@ -413,7 +414,7 @@ quoted_printable_encode(infile, outfile, text, length)
 	FILE *outfile;
 	int text, length;
 {
-    int c, len = 0, sp = OFF;
+    int c, len = 0, sp = OFF, lfdot = OFF;
     while ((c = getc(infile)) != EOF) {
 	if ((c == TAB) || (c == SP)) {
 	    sp = ON;
@@ -421,14 +422,16 @@ quoted_printable_encode(infile, outfile, text, length)
 	    if ((++len) > length) {
 		sp = OFF;
 		len = 0;
+		lfdot = LF;
 		softbreak(outfile);
 	    }
 	    continue;
-	}
+        }
 	if (c == LF) {
-	    if (sp) softbreak(outfile);
+	    if (sp || (lfdot == DOT)) softbreak(outfile);
 	    len = 0;
-	    sp = OFF;    
+	    sp = OFF;
+	    lfdot = LF;
 	    putc(LF, outfile);
 	    continue;
 	}
@@ -440,16 +443,22 @@ quoted_printable_encode(infile, outfile, text, length)
 	    len += 3;
 	    if (len > length) {
 		len = 0;
+		lfdot = LF;
 		softbreak(outfile);
+	    } else {
+		lfdot = OFF;
 	    }
 	    continue;
-	}
+        }
 	sp = OFF;
+	if (c == DOT)
+	    if (lfdot == LF) lfdot = DOT; else lfdot = OFF;
 	putc(c, outfile);
 	if ((++len) > length) {
 	    len = 0;
+	    lfdot = LF;
 	    softbreak(outfile);
-	}
+        }
     }
     if (len > 0) softbreak(outfile);  /* ignored by decoder */
 }
@@ -469,16 +478,20 @@ puthexchar(c1, c2, stream)
 	int c1, c2;
 	FILE *stream;
 {
-    if ((c1 = base128[c1]) == OOB) {
-	fprintf(stderr, "Error: can't translate hex to character.\n");
-	exit(ERROR);
+    int a1, a2;
+    static int warned = OFF;
+
+    if (((a1 = base128[c1]) != OOB) && ((a2 = base128[c2]) != OOB)) {
+	putc(((a1 << 4) | a2), stream);
+	return (SUCCESS);
+    } else {
+	if (warned == OFF) { /* warn just once */
+            fprintf(stderr, "Error: can't translate hex to character: %c%c\n",
+		    c1, c2);
+	    warned = ON;
+        }
+	return (ERROR);	
     }
-    if ((c2 = base128[c2]) == OOB) {
-	fprintf(stderr, "Error: can't translate hex to character.\n");
-	exit(ERROR);
-    }
-    putc(((c1 << 4) | c2), stream);
-       
 }
 
 quoted_printable_decode(infile, outfile, text, length)
@@ -488,19 +501,30 @@ quoted_printable_decode(infile, outfile, text, length)
 {
     int c1, c2, c3;
 
+    /* if an error occurs, print input sequence as it is, anyway, sigh */
+    
     while((c1 = getc(infile)) != EOF) {
+    skipgetc:
 	if (c1 == EQ) {
 	    if ((c2 = getc(infile)) == EOF) {
-		fprintf(stderr, "Error End of file after =.\n");
+		fprintf(stderr, "Error: end of file after =.\n");
+		putc(EQ, outfile);
 		exit(ERROR);
 	    }
 	    if (c2 == LF) continue;
 	    if ((c3 = getc(infile)) == EOF) {
-		fprintf(stderr, "Error End of file after =.\n");
+		fprintf(stderr, "Error: end of file after =.\n");
+		putc(EQ, outfile);
+		putc(c2, outfile);
 		exit(ERROR);
 	    }
-	    puthexchar(c2, c3, outfile);
-	    continue;
+	    if (puthexchar(c2, c3, outfile) == ERROR) {
+		putc(EQ, outfile);
+		if ((c1 = c2) == EOF) exit(ERROR);
+		ungetc(c3, infile);
+		goto skipgetc;
+	    } else
+	       continue;
 	}
 	putc(c1, outfile);
     }
@@ -558,7 +582,7 @@ gzip64_encode(infile, outfile, text, length)
 	    dup(fileno(infile));
 	}
 	
-	execlp("gzip", "gzip", (char *) 0);
+	execlp("gzip", "gzip", "-n", "-c", (char *) 0);
 	
 	exit(SUCCESS);
 
@@ -584,7 +608,7 @@ gzip64_encode(infile, outfile, text, length)
 	    close(READ);
 	    dup(pipes2[READ]);
 
-	    execlp("gzip", "gzip", (char *) 0);
+	    execlp("gzip", "gzip", "-n", "-c", (char *) 0);
 	    
 	    exit(SUCCESS);
 	}
@@ -641,7 +665,7 @@ gzip64_decode(infile, outfile, text, length)
 		close(WRITE);
 		dup(fileno(outfile));
 	    }
-	    execlp ("gzip", "gzip", "-d", (char *) 0);
+	    execlp ("gzip", "gzip", "-d", "-n", "-c", (char *) 0);
 	}
 	
 	exit(SUCCESS);
@@ -675,7 +699,7 @@ gzip64_decode(infile, outfile, text, length)
 	    close(READ);
 	    dup(pipes2[READ]);
 
-	    execlp("gzip", "gzip", "-d", (char *) 0);
+	    execlp("gzip", "gzip", "-d", "-n", "-c", (char *) 0);
 	
 	    exit(SUCCESS);	    
 	}
@@ -684,10 +708,17 @@ gzip64_decode(infile, outfile, text, length)
 
 	close(pipes2[READ]);
 
-	if ((outfile = fdopen(pipes2[WRITE], "w")) == NULL) {
+#if defined(OS2)
+	if ((outfile = fdopen(pipes2[WRITE], "wb")) == NULL) {
 	    fprintf(stderr, "Can't open write pipe\n");
 	    exit(ERROR);
 	}
+#else
+ 	if ((outfile = fdopen(pipes2[WRITE], "w")) == NULL) {
+  	    fprintf(stderr, "Can't open write pipe\n");
+  	    exit(ERROR);
+  	}
+#endif
 	
 	base64_decode(infile, outfile, OFF, length);
 
@@ -695,11 +726,17 @@ gzip64_decode(infile, outfile, text, length)
 
     } else {
 
-	if ((outfile = fdopen(pipes[WRITE], "w")) == NULL) {
+#if defined(OS2)
+	if ((outfile = fdopen(pipes[WRITE], "wb")) == NULL) {
 	    fprintf(stderr, "Can't open write pipe\n");
 	    exit(ERROR);
 	}
-	
+#else
+ 	if ((outfile = fdopen(pipes[WRITE], "w")) == NULL) {
+  	    fprintf(stderr, "Can't open write pipe\n");
+  	    exit(ERROR);
+  	}
+#endif	
 	base64_decode(infile, outfile, OFF, length);
 	
 	exit(SUCCESS);
@@ -718,7 +755,12 @@ int main (argc, argv)
     FILE *infile;
     FILE *outfile;
     int file_count = 0;
+#if defined(OS2)
+    char prognametmp[256];
+    char *progname;
+#else
     char *progname = basename(argv[0]);
+#endif	
 
     /*
      * default option values
@@ -730,6 +772,11 @@ int main (argc, argv)
     int  text   = OFF;			/* -t */
 
 
+#if defined(OS2)
+    strcpy(prognametmp, argv[0]);
+    strlwr(prognametmp);
+    progname = basename(prognametmp);
+#endif	
     while((optc = getopt_long(argc, argv, "dbqgzl:thv", longopts,
 			      (int *)0)) != EOF) {
 	switch (optc) {
@@ -768,17 +815,21 @@ int main (argc, argv)
 
     file_count = argc - optind;
 
+
+#if defined(OS2)
+    _fsetmode(stdout, "b");
+#endif
     switch(file_count) {
     case 0:
 	infile  = stdin;
-	outfile = stdout;
+ 	outfile = stdout;
 	break;
     case 1:
 	if ((infile  = fopen(argv[optind], "r")) == NULL) {
 	    fprintf(stderr, "Can't open file %s.\n", argv[optind]);
 	    exit(ERROR);
 	}
-	outfile = stdout;
+ 	outfile = stdout;
 	break;
     case 2:
 	if ((infile  = fopen(argv[optind], "r")) == NULL) {
@@ -786,10 +837,17 @@ int main (argc, argv)
 	    exit(ERROR);
 	}
 	optind++;
-	if ((outfile = fopen(argv[optind], "w")) == NULL) {
+#if defined(OS2)
+	if ((outfile = fopen(argv[optind], "wb")) == NULL) {
 	    fprintf(stderr, "Can't open file %s.\n", argv[optind]);
 	    exit(ERROR);
 	}
+#else
+ 	if ((outfile = fopen(argv[optind], "w")) == NULL) {
+  	    fprintf(stderr, "Can't open file %s.\n", argv[optind]);
+  	    exit(ERROR);
+  	}
+#endif
 	break;
     default:
 	usage(progname);
@@ -798,13 +856,20 @@ int main (argc, argv)
     }
 
     /* Override argments by progname. */
-    
-    if (strcmp(progname, DECODE_NAME) == 0) decode = ON;
-    if (strcmp(progname, CAT_NAME) == 0) {
+
+#if defined(OS2)
+    if (strncmp(progname, DECODE_NAME, sizeof(DECODE_NAME)) != 0) decode = ON;
+    if (strncmp(progname, CAT_NAME, sizeof(CAT_NAME)) != 0) {
 	decode = ON;
 	outfile = stdout;
     }
-
+#else
+     if (strcmp(progname, DECODE_NAME) == 0) decode = ON;
+     if (strcmp(progname, CAT_NAME) == 0) {
+  	decode = ON;
+  	outfile = stdout;
+      }
+#endif
 
     if (decode) {
 	switch (encode) {
