@@ -1,16 +1,20 @@
 ;;;			      mew-pgp.el
 ;;;
-;;;	     Copyright (C) 1994, 1995  Kazuhiko Yamamoto
+;;;	  Copyright (C) 1994, 1995, 1996  Kazuhiko Yamamoto
 ;;;
 ;;;		   This emacs lisp library conforms
 ;;;		GNU GENERAL PUBLIC LICENSE Version 2.
 ;;;
 ;;; Author:  Kazuiko Yamamoto <kazu@is.aist-nara.ac.jp>
-;;; Created: August 17, 1994
-;;; Revised: March   4, 1995
+;;; Created: August   17, 1994
+;;; Revised: February 29, 1996
 ;;;
 
-(defconst mew-pgp-version "mew-pgp.el version 0.12")
+(defconst mew-pgp-version "mew-pgp.el version 0.17")
+
+(if mew-emacs19-p
+    (eval-when-compile 
+      (require 'terminal)))
 
 (provide 'mew-pgp)
 (require 'mew)
@@ -26,9 +30,23 @@
   "application/pgp")
 
 (defvar mew-pgp-encode-switch 
-  '((signature  . "-staf +clearsig=on +language=en")
-    (encryption . "-etaf +language=en")
-    (sign-enc   . "-estaf +language=en"))
+  '((pgp-sign  . "-staf +clearsig=on +language=en")
+    (pgp-enc . "-etaf +language=en")
+    (pgp-sign-enc   . "-estaf +language=en"))
+  )
+
+;; temporary solution
+
+(defvar mew-pgp-encode-text-switch 
+  '((pgp-sign  . "-staf +clearsig=on +language=en")
+    (pgp-enc . "-etaf +language=en")
+    (pgp-sign-enc   . "-estaf +language=en"))
+  )
+
+(defvar mew-pgp-encode-binary-switch 
+  '((pgp-sign  . "-saf +clearsig=on +language=en")
+    (pgp-enc . "-eaf +language=en")
+    (pgp-sign-enc   . "-esaf +language=en"))
   )
 
 (defvar mew-pgp-decode-args "-f +language=en")
@@ -82,23 +100,38 @@
 
 (defun mew-pgp-sign-letter ()
   (interactive)
-  (if (mew-field-get-value "Content-Type:")
-      (message "You can't sign this draft due to Content-Type:")
-    (mew-pgp-encode-letter 'signature))
+  (cond
+   ((mew-field-get-value "Content-Type:")
+    (message "You can't sign this draft due to Content-Type:"))
+   ((mew-multi-p)
+    (message "You can't sign this draft with the shortcut method"))
+   (t
+    (mew-pgp-encode-letter 'pgp-sign))
+   )
   )
 
 (defun mew-pgp-encrypt-letter ()
   (interactive)
-  (if (mew-field-get-value "Content-Type:")
-      (message "You can't encrypt this draft due to Content-Type:")
-    (mew-pgp-encode-letter 'encryption))
+  (cond
+   ((mew-field-get-value "Content-Type:")
+    (message "You can't encrypt this draft due to Content-Type:"))
+   ((mew-multi-p)
+    (message "You can't encrypt this draft with the shortcut method"))
+   (t
+    (mew-pgp-encode-letter 'pgp-enc))
+   )
   )
 
 (defun mew-pgp-sign-encrypt-letter ()
   (interactive)
-  (if (mew-field-get-value "Content-Type:")
-      (message "You can't sign then ecrypt this draft due to Content-Type:")
-    (mew-pgp-encode-letter 'sign-enc))
+  (cond 
+   ((mew-field-get-value "Content-Type:")
+    (message "You can't sign then ecrypt this draft due to Content-Type:"))
+   ((mew-multi-p)
+    (message "You can't sign then ecrypt this draft with the shortcut method"))
+   (t 
+    (mew-pgp-encode-letter 'pgp-sign-enc))
+   )
   )
 
 (defun mew-pgp-encode-letter (action)
@@ -126,7 +159,7 @@
     (if (null (mew-which mew-prog-pgp exec-path))
 	(message "pgp is not found")
       (cond
-       ((or (equal action 'encryption) (equal action 'sign-enc))
+       ((or (equal action 'pgp-enc) (equal action 'pgp-sign-enc))
 	(if (setq users 
 		  (mew-header-canform-list
 		   (mew-header-expand-alias-list
@@ -138,15 +171,15 @@
 			  ))
 	  (error "Specify To: or Cc:")
 	  ))
-       ((equal action 'signature)
+       ((equal action 'pgp-sign)
 	(setq users "")
 	)
        (t (error "No such action."))
        )
-      (setq mew-pgp-tmp-read-file (make-temp-name "/tmp/mew"))
+      (setq mew-pgp-tmp-read-file (make-temp-name mew-temp-file))
       (write-region beg end mew-pgp-tmp-read-file)
       ;; touch file before write-file to make them unique
-      (setq mew-pgp-tmp-write-file (make-temp-name "/tmp/mew"))
+      (setq mew-pgp-tmp-write-file (make-temp-name mew-temp-file))
       (setq mew-pgp-tmp-begin beg)
       (setq mew-pgp-tmp-end end)
       (message "PGP encoding ...")
@@ -160,8 +193,8 @@
 				    users
 				    mew-pgp-tmp-read-file
 				    mew-pgp-tmp-write-file)))
-      ;; I seems to me that PGP on UNIX can't handle ^D from stdio.
-      ;; So, tmpfile1.
+      ;; It seems to me that PGP on UNIX can't handle ^D from stdio.
+      ;; So, tmp-read-file.
       (set-process-filter process 'mew-pgp-encode-filter1)
       (set-process-sentinel process 'mew-pgp-encode-sentinel)
       ;; Wait for the termination of PGP.
@@ -213,10 +246,11 @@
 (defun mew-pgp-encode-filter1 (process string)
   (cond
    ((string-match "Enter pass phrase: $" string) ;; sign or sign-enc
+    (set-process-filter process 'mew-pgp-encode-filter2)
     (process-send-string
      process
      (format "%s\n" (mew-read-passwd "Enter pass phrase : ")))
-    (set-process-filter process 'mew-pgp-encode-filter2))
+    )
    ((string-match "Keyring file" string) ;; no secring
     (setq mew-pgp-encode-failure 'nofile)
     (set-process-filter process 'mew-pgp-encode-filter3))
@@ -274,7 +308,7 @@
 
 (defvar mew-pgp-decode-failure nil)
 
-(defun mew-pgp-decode-region (beg end)
+(defun mew-pgp-decode-region (beg end &optional mime)
   (interactive "r")
   (if (mew-which mew-prog-pgp exec-path)
       (let ((process)
@@ -282,16 +316,17 @@
 	    ;; you aren't expected to understand this.
 	    (default-process-coding-system
 	      (if mew-mule-p
-		  (cons mew-mule-charset-integrity
-			mew-mule-charset-integrity)
-		nil))
+		  (if mime
+		      (cons *noconv* *noconv*)
+		    (cons mew-mule-charset-integrity
+			  mew-mule-charset-integrity))))
 	    (file-coding-system
 	     (if mew-mule-p mew-mule-charset-integrity))
 	    )
-	(setq mew-pgp-tmp-read-file (make-temp-name "/tmp/mew"))
+	(setq mew-pgp-tmp-read-file (make-temp-name mew-temp-file))
 	(write-region beg end mew-pgp-tmp-read-file)
 	;; touch file before write-file to make them unique
-	(setq mew-pgp-tmp-write-file (make-temp-name "/tmp/mew"))
+	(setq mew-pgp-tmp-write-file (make-temp-name mew-temp-file))
 	(message "PGP decoding ... ")
 	(setq process
 	      (start-process "PGP decode"
@@ -302,10 +337,12 @@
 				     mew-pgp-decode-args
 				     mew-pgp-tmp-read-file
 				     mew-pgp-tmp-write-file)))
-	;; I seems to me that PGP on UNIX can't handle ^D from stdio.
+	;; It seems to me that PGP on UNIX can't handle ^D from stdio.
 	;; So, read-file.
 	(set-process-filter process 'mew-pgp-decode-filter1)
-	(set-process-sentinel process 'mew-pgp-decode-sentinel)
+	(if mime
+	    (set-process-sentinel process 'mew-pgp-decode-sentinel-mime)
+	  (set-process-sentinel process 'mew-pgp-decode-sentinel))
 	(delete-region beg end)
 	;; Wait for the termination of PGP.
 	;; Emacs doesn't provide synchronize mechanism with
@@ -318,33 +355,59 @@
 	)
     ))
 
-(defun mew-pgp-decode-sentinel (process event)
+(defun mew-pgp-decode-sentinel-mime (process event)
+  (mew-pgp-decode-sentinel process event 'mime))
+
+(defun mew-pgp-decode-sentinel (process event &optional mime)
   (save-excursion
     (let ((file-coding-system-for-read
-	   (if mew-mule-p mew-mule-charset-integrity)))
+	   (if mew-mule-p (if mime *noconv* mew-mule-charset-integrity)))
+	  (msg (or mew-pgp-decode-message ""))
+	  (message ""))
       (set-buffer (process-buffer process))
       (if mew-pgp-decode-failure
 	  (cond
-;;	   ((eq mew-pgp-decode-failure 'nofile)
-;;	    (setq mew-pgp-decode-message "pubring doesn't exist."))
+	   ((eq mew-pgp-decode-failure 'nofile)
+	    (setq message "PGP keyring doesn't exist"))
+	   ((eq mew-pgp-decode-failure 'nopubkey)
+	    (setq message "Can't find the public key"))
 	   ((eq mew-pgp-decode-failure 'nouserid)
-	    (setq mew-pgp-decode-message
-		  "\nThe userid not found in pubring\n"))
+	    (setq message "The userid not found in PGP pubring"))
 	   ((eq mew-pgp-decode-failure 'nopass)
-	    (setq mew-pgp-decode-message "\nPass phrase is wrong.\n"))
-	   (t (setq mew-pgp-decode-message
-		    "Decoding failed sor some reasons."))
-	   )
-	(insert-file-contents mew-pgp-tmp-write-file)
-	(goto-char (point-max))
-	)
-      (insert "\n")
-      (insert mew-pgp-report-boundary1)
-      (if mew-pgp-decode-message
-	  (insert mew-pgp-decode-message)
-	(insert mew-pgp-encrypt-only-message))
-      (insert mew-pgp-report-boundary2)
-      ;;
+	    (setq message "Pass phrase is wrong"))
+	   (t
+	    (setq message "Decoding failed sor some reasons."))
+	   ))
+      (insert-file-contents mew-pgp-tmp-write-file) 
+      (if mime
+	  ;; delete CR to set line-break as LF
+	  (catch 'cr
+	    (while (looking-at ".*\r$")
+	      (end-of-line)
+	      (delete-char -1)
+	      (if (bolp) (throw 'cr nil))
+	      (forward-line)
+	      ))
+	(forward-line -1)) ;; before null line
+      (if (string-match "PGP decrypted" msg)
+	  (setq message (concat message "PGP decrypted. ")))
+      (if (string-match "\n\\(.*\\) signature from user \\(.*\\)\\." msg)
+	  (progn
+	    (setq message (concat message (mew-match 1 msg) " PGP sign "
+				  (mew-match 2 msg)))
+	    (cond
+	     ((string-match "not certified with enough" msg)
+	      (setq message (concat message " MARGINAL")))
+	     ((string-match "not trusted" msg)
+	      (setq message (concat message " UNTRUSTED")))
+	     ;; PGP uses "unknown" for validity internally, but
+	     ;; prints "undefined" for "unknown".
+	     ((string-match "not certified with a" msg)
+	      (setq message (concat message " UNDEFINED")))
+	     (t
+	      (setq message (concat message " COMPLETE")))
+	     )))
+      (insert (concat mew-x-mew " " message "\n"))
       (setq mew-pgp-decode-message nil)
       (if (file-exists-p mew-pgp-tmp-read-file)
 	  (delete-file mew-pgp-tmp-read-file))
@@ -360,17 +423,17 @@
 (defun mew-pgp-decode-filter1 (process string)
   (cond
    ((string-match "Enter pass phrase: $" string)
-    ;; decrypt, decrypt then verfy
+    ;; decrypt(1), decrypt then verfy(2)
+    (set-process-filter process 'mew-pgp-decode-filter2)
     (process-send-string
      process
      (format "%s\n" (mew-read-passwd "Enter pass phrase : ")))
-    (set-process-filter process 'mew-pgp-decode-filter2))
+    )
    ((string-match "You do not have the secret key" string)
     ;; this includes not secring.pgp
     (setq mew-pgp-decode-failure 'nouserid)
     (set-process-filter process 'mew-pgp-decode-filter4))
-   ;; how about "secring.pgp doesn't exist"?  'nofile
-   ((string-match "File has signature" string) ;; just verfy
+   ((string-match "File has signature" string) ;; verfy(3)
     (setq mew-pgp-decode-message (concat mew-pgp-decode-message string))
     (set-process-filter process 'mew-pgp-decode-filter3))
    )
@@ -383,10 +446,11 @@
     (setq mew-pgp-decode-failure 'nopass)
     (set-process-filter process 'mew-pgp-decode-filter4))
    ((string-match "Pass phrase is good." string) 
-    ;; decrypt, decrypt then verfy
+    ;; decrypt(1), decrypt then verfy(2)
     (message "PGP decoding ... ")
-    (set-process-filter process 'mew-pgp-decode-filter3)
-    )
+    (setq mew-pgp-decode-message (concat mew-pgp-decode-message
+					 "PGP decrypted "))
+    (set-process-filter process 'mew-pgp-decode-filter3))
    ((string-match "Error:  Bad pass phrase.\n\nEnter pass phrase: $" string)
     (process-send-string 
      process
@@ -401,11 +465,18 @@
     (set-buffer (process-buffer process))
     (cond
      ((string-match "Just a moment" string)
-      ;; decrypt, decrypt then verfy
-      ());; do nothing
-     ((string-equal "." string) ;; decrypt
+      ;; decrypt(1), decrypt then verfy(2)
       ()) ;; do nothing
-     (t ;; decrypt then verfy
+     ((string-equal "." string)
+      ;; decrypting - decrypt(1), decrypt then verfy(2)
+      ()) ;; do nothing
+     ((string-match "Can't find the right public key" string)
+      (setq mew-pgp-decode-failure 'nopubkey)
+      (set-process-filter process 'mew-pgp-decode-filter4))
+     ((string-match "Keyring file.* does not exist" string)
+      (setq mew-pgp-decode-failure 'nofile)
+      (set-process-filter process 'mew-pgp-decode-filter4))
+     (t ;; verfy message - decrypt then verfy(2), verfy(3)
       (setq mew-pgp-decode-message (concat mew-pgp-decode-message string))
       )
      )
@@ -435,7 +506,7 @@
 (defun mew-pgp-add-key ()
   (interactive)
   (mew-summary-display 'notforce)
-  (setq mew-pgp-tmp-read-file (make-temp-name "/tmp/mew"))
+  (setq mew-pgp-tmp-read-file (make-temp-name mew-temp-file))
   (save-excursion
     (set-buffer (mew-buffer-message))
     (save-restriction
@@ -524,6 +595,148 @@
 	)
   )
 
+;;
+;; PGP/MIME
+;;
+
+(defun mew-pgp-user-list (string)
+  (let ((list))
+    (setq list (mew-header-canform-list
+		(mew-header-expand-alias-list
+		 (mew-header-delete-nullstring-list
+		  (mew-header-extract-addr-list
+		   (mew-header-split
+		    (mew-header-concat-list
+		     (mew-header-delete-ws2-list
+		      (mew-header-separate-quote string)))
+		    ?,))))))
+    (concat (mapconcat (function (lambda (x) x)) list " ")
+	    ;; list to string
+	    " " mew-mail-address ;; to decrypt by myself
+	    )
+    ))
+
+(defun mew-insert-pgp-encoded-file (file type desc encoding param)
+  ;; desc may be nil not ""
+  (let ((case-fold-search t)
+	buf towhom options
+	file-coding-system
+	file-coding-system-for-read
+	text process)
+    ;;
+    ;; Treat as text?
+    ;;
+    (let ((case-fold-search t))
+      (if (or (string-match "^Text" type)
+	      (string-match "^Multipart" type)
+	      (string-match "^Message" type)
+	      (string-match "^Application/Postscript$" type))
+	  (setq text t)
+	(setq text nil)))
+    ;;
+    ;; Prepare read-file
+    ;;
+    (if (string-match "^Multipart" type)
+	(setq mew-pgp-tmp-read-file file)
+      (save-excursion
+	(setq mew-pgp-tmp-read-file (make-temp-name mew-temp-file))
+	(setq buf (find-file-noselect mew-pgp-tmp-read-file))
+	(set-buffer buf)
+	(if mew-mule-p
+	    (if (string-match "^Text" type)
+		(setq file-coding-system mew-mule-charset-integrity
+		      file-coding-system-for-read mew-mule-charset-integrity)
+	      (setq file-coding-system *noconv*
+		    file-coding-system-for-read *noconv*)))
+	(insert-file-contents file)
+	(goto-char (point-min))
+	(if (not (string-match "^Text" type))
+	    (mew-field-insert-here "Content-Type:" type (not text))
+	  (mew-field-insert-here
+	   "Content-Type:"
+	   (concat type "; charset="		 
+		   (mew-draft-charset-guess-region (point) (point-max)))
+	   (not text)))
+	(if desc (mew-field-insert-here "Content-Description:"
+					(concat "\"" desc "\"") (not text)))
+	(if text (insert "\n") (insert "\r\n"))
+	(save-buffer)
+	(kill-buffer buf))
+      )
+    ;;
+    ;; prepare user ids
+    ;;
+    (if (or (equal encoding 'pgp-enc)
+	    (equal encoding 'pgp-sign-enc))
+	;; CD: is used as user list, oh my!!
+	(setq towhom (mew-pgp-user-list param)) 
+      (setq towhom ""))
+    (if text
+	(setq options (cdr (assoc encoding mew-pgp-encode-text-switch)))
+      (setq options (cdr (assoc encoding mew-pgp-encode-binary-switch))))
+    (setq mew-pgp-tmp-write-file (make-temp-name mew-temp-file))
+    (setq process
+	  (start-process "PGP encode"
+			 (current-buffer)
+			 mew-prog-shell "-c"
+			 (format "%s %s %s < %s > %s"
+				 mew-prog-pgp
+				 options
+				 towhom
+				 mew-pgp-tmp-read-file
+				 mew-pgp-tmp-write-file)))
+    ;; It seems to me that PGP on UNIX can't handle ^D from stdio.
+    ;; So, read-file
+    (set-process-filter process 'mew-pgp-encode-filter1)
+    (set-process-sentinel process 'mew-insert-pgp-encoded-file-sentinel)
+    ;; Wait for the termination of PGP.
+    ;; Emacs doesn't provide synchronize mechanism with
+    ;; an asynchronous process. So, take this way. 
+    (while (not (equal (process-status process) 'exit))
+      (sit-for 1) ;; give CPU time to the process filter 
+      ;; accept-process-output or sleep-for is not enough
+      )
+    (if mew-pgp-encode-failure
+	(progn
+	  (setq mew-pgp-encode-failure nil)
+	  nil) ;; return
+      (message "PGP encoding ... done")
+      t) ;; return
+    ))
+
+(defun mew-insert-pgp-encoded-file-sentinel (process event)
+  (save-excursion
+    (let ((file-coding-system-for-read
+	   (if mew-mule-p mew-mule-charset-integrity)))
+      (set-buffer (process-buffer process))
+      (if mew-pgp-encode-failure
+	  (cond
+	   ((eq mew-pgp-encode-failure 'nofile)
+	    (message "secring doesn't exist."))
+	   ((eq mew-pgp-encode-failure 'nouserid)
+	    (message "userid %s is not found in secring." (user-login-name)))
+	   ((eq mew-pgp-encode-failure 'nopass)
+	    (message "Pass phrase is wrong"))
+	   (t (message "Encoding failed for some reasons"))
+	   )
+	(mew-field-insert-here "Content-Type:" 
+			       (concat mew-pgp-content-type "; format=mime"))
+	(insert "\n")
+	(insert-file-contents mew-pgp-tmp-write-file))
+      ;;
+      (if (file-exists-p mew-pgp-tmp-read-file)
+	  (delete-file mew-pgp-tmp-read-file))
+      (if (file-exists-p mew-pgp-tmp-write-file)
+	  (delete-file mew-pgp-tmp-write-file))
+      (setq mew-pgp-tmp-read-file nil)
+      (setq mew-pgp-tmp-write-file nil)
+;      (setq mew-pgp-tmp-begin nil)
+;      (setq mew-pgp-tmp-end nil)
+      )
+    ))
+
+
+
 ;;
 ;; End of file
 ;;
