@@ -8,10 +8,10 @@
 ;;; Author:  Mine Sakurai <m-sakura@ccs.mt.nec.co.jp>
 ;;;          Kazuhiko Yamamoto <kazu@is.aist-nara.ac.jp>
 ;;; Created: June     8, 1994
-;;; Revised: February 7, 1995
+;;; Revised: Sep 28, 1995
 ;;;
 
-(defconst mew-url-version "mew-url.el version 0.17")
+(defconst mew-url-version "mew-url.el version 0.18")
 
 (provide 'mew-url)
 (require 'mew)
@@ -362,7 +362,6 @@
 	   (save-excursion
 	     (unwind-protect
 		 (let ((to server)
-		       (cc "")
 		       (subject (if subject-param subject-param 
 				  "Automated Mail Server Request"))
 		       (file (mew-draft-get-new))
@@ -370,7 +369,7 @@
 		   (switch-to-buffer (find-file-noselect file))
 		   (mew-draft-mode)
 		   (mew-draft-rename file)
-		   (mew-draft-header to cc subject)
+		   (mew-draft-header subject nil to nil)
 		   (insert-buffer-substring (mew-current-get 'cache) p end)
 		   )
 	       (save-buffer))
@@ -509,7 +508,8 @@
 		     (mew-file-content 
 		      (mew-file-attr file mew-mime-content-type))
 		     ""
-		     nil)
+		     nil
+		     "")
 	       mew-multi-syntax))
 	(mew-multi-display)
 	))
@@ -837,6 +837,7 @@ mandatory is t if the parameter is mandatory for the access-type specific."
     (setq all (ange-ftp-file-name-all-completions file path))
     (cond
      ((eq comp t) 
+      (insert file)
       (and (get-buffer mew-buffer-completions)
 	   (kill-buffer mew-buffer-completions))
       )
