@@ -5,9 +5,9 @@
 ;;;		   This emacs lisp library confirms
 ;;;		GNU GENERAL PUBLIC LICENSE Version 2.
 ;;;
-;;; Author:  Yoshinari NOMURA <nomsun@csce.kyushu-u.ac.jp>
+;;; Author:  Yoshinari NOMURA <nom@csce.kyushu-u.ac.jp>
 ;;; Created: Jul    4, 1994
-;;; Revised: Dec   17, 1994
+;;; Revised: Nov   16, 1995
 
 ;;;
 ;;; Constants.
@@ -15,7 +15,7 @@
 
 (defconst mhc-date-field "X-SC-Date:")
 (defconst mhc-subject-field "X-SC-Subject:")
-(defconst mhc-mode-version "mhc.el version 0.08")
+(defconst mhc-mode-version "mhc.el version 0.09")
 (defconst mhc-rfc-date-re
   (concat 
    "^ *"
@@ -36,7 +36,7 @@
 (defvar mhc-guess-date-re '("\\([$B#0(B-$B#9(B0-9]+\\) *[$B!?(B/$B7n(B] *\\([$B#0(B-$B#9(B0-9]+\\)"))
 (defvar mhc-guess-time-re '("\\([$B#0(B-$B#9(B0-9]+\\) *[$B!'(B:$B;~(B] *\\([$B#0(B-$B#9(B0-9]+\\)"))
 (defvar mhc-mailer-package 'mh-e)
-(defvar mhc-scan-function 'mh-visit-folder)
+(defvar mhc-scan-function 'mhc-scan-function-for-mew)
 (defvar mhc-mail-path (expand-file-name "~/Mail"))
 (defvar mhc-schedule-folder "schedule")
 (defvar mhc-window-stack nil)
@@ -62,7 +62,7 @@
 (defun mhc-mode (&optional args)
   (interactive)
   (cond ((equal mhc-mailer-package 'mew)
-	 (setq mhc-scan-function 'mew-summary-scan))
+	 (setq mhc-scan-function 'mhc-scan-function-for-mew))
 	((equal mhc-mailer-package 'mh-e)
 	 (setq mhc-scan-function 'mh-visit-folder))
 	(t
@@ -115,8 +115,7 @@
   (let ((yy (nth 0 (mhc-get-digit-date-today)))
 	(mm (nth 1 (mhc-get-digit-date-today))))
     (funcall mhc-scan-function
-	     (format "+%s/%02d/%02d" mhc-schedule-folder yy mm) 
-	     (list "all" t))))
+	     (format "+%s/%02d/%02d" mhc-schedule-folder yy mm))))
 
 (defun mhc-goto-next-month ()
   (interactive)
@@ -127,8 +126,7 @@
       (setq mm (1+ mm)))
     (if (mhc-parse-digit-date (list yy mm '1 '0 '0))
 	(funcall mhc-scan-function
-		 (format "+%s/%02d/%02d" mhc-schedule-folder yy mm) 
-		 (list "all" t)))
+		 (format "+%s/%02d/%02d" mhc-schedule-folder yy mm) ))
     ))
 
 (defun mhc-goto-previous-month ()
@@ -140,10 +138,16 @@
       (setq mm (1- mm)))
     (if (mhc-parse-digit-date (list yy mm '1 '0 '0))
 	(funcall mhc-scan-function
-	       (format "+%s/%02d/%02d" mhc-schedule-folder yy mm)
-	       (list "all" t)))
+	       (format "+%s/%02d/%02d" mhc-schedule-folder yy mm)))
     ))
 
+(defun mhc-scan-function-for-mew (folder)
+  (if (get-buffer folder)
+      (switch-to-buffer folder)
+    (mew-summary-folder-create folder))
+  (mew-summary-scan-body folder (list "all" 'erase))
+  )
+
 ;;
 ;; Entry, modify
 ;;
@@ -647,3 +651,4 @@
       (set-window-configuration (car-safe mhc-window-stack))
     (message "Window Stack is empty."))
   (setq mhc-window-stack (cdr-safe mhc-window-stack)))
+
